@@ -11,7 +11,7 @@ import traceback
 from numba import (bytecode, interpreter, funcdesc, typing, typeinfer,
                    lowering, objmode, irpasses, utils, config,
                    types, ir, looplifting, macro, types, rewrites)
-from numba.targets import cpu
+from numba.targets import cpu, callconv
 from numba.annotations import type_annotations
 
 
@@ -19,24 +19,25 @@ class Flags(utils.ConfigOptions):
     # These options are all false by default, but the defaults are
     # different with the @jit decorator (see targets.options.TargetOptions).
 
-    OPTIONS = frozenset([
+    OPTIONS = {
         # Enable loop-lifting
-        'enable_looplift',
+        'enable_looplift': False,
         # Enable pyobject mode (in general)
-        'enable_pyobject',
+        'enable_pyobject': False,
         # Enable pyobject mode inside lifted loops
-        'enable_pyobject_looplift',
+        'enable_pyobject_looplift': False,
         # Force pyobject mode inside the whole function
-        'force_pyobject',
+        'force_pyobject': False,
         # Release GIL inside the native function
-        'release_gil',
-        'no_compile',
-        'boundcheck',
-        'forceinline',
-        'no_cpython_wrapper',
-        'nrt',
-        'no_rewrites',
-    ])
+        'release_gil': False,
+        'no_compile': False,
+        'boundcheck': False,
+        'forceinline': False,
+        'no_cpython_wrapper': False,
+        'nrt': False,
+        'no_rewrites': False,
+        'error_model': 'python',
+    }
 
 
 DEFAULT_FLAGS = Flags()
@@ -66,7 +67,7 @@ class CompileResult(namedtuple("_CompileResult", CR_FIELDS)):
         """
         Reduce a CompileResult to picklable components.
         """
-        libdata = self.library.serialize()
+        libdata = self.library.serialize_using_object_code()
         # Make it (un)picklable efficiently
         typeann = str(self.type_annotation)
         fndesc = self.fndesc
@@ -79,7 +80,7 @@ class CompileResult(namedtuple("_CompileResult", CR_FIELDS)):
     @classmethod
     def _rebuild(cls, target_context, libdata, fndesc, env,
                  signature, objectmode, interpmode, lifted, typeann):
-        library = target_context.jit_codegen().unserialize_library(libdata)
+        library = target_context.codegen().unserialize_library(libdata)
         cfunc = target_context.get_executable(library, fndesc, env)
         cr = cls(target_context=target_context,
                  typing_context=target_context.typing_context,
@@ -275,6 +276,8 @@ class Pipeline(object):
             subtargetoptions['enable_boundcheck'] = True
         if flags.nrt:
             subtargetoptions['enable_nrt'] = True
+        error_model = callconv.error_models[flags.error_model](targetctx.call_conv)
+        subtargetoptions['error_model'] = error_model
 
         self.targetctx = targetctx.subtarget(**subtargetoptions)
         self.library = library
@@ -528,8 +531,12 @@ class Pipeline(object):
         Back-end: Generate LLVM IR from Numba IR, compile to machine code
         """
         if self.library is None:
-            codegen = self.targetctx.jit_codegen()
+            codegen = self.targetctx.codegen()
             self.library = codegen.create_library(self.bc.func_qualname)
+            # Enable object caching upfront, so that the library can
+            # be later serialized.
+            self.library.enable_object_caching()
+
         lowered = lowerfn()
         signature = typing.signature(self.return_type, *self.args)
         cr = compile_result(typing_context=self.typingctx,
