@@ -3,7 +3,9 @@ import inspect
 
 from . import _internal, dufunc
 from .ufuncbuilder import UFuncBuilder, GUFuncBuilder
+from .parallel import ParallelUFuncBuilder
 
+from numba.cuda.vectorizers import CUDAVectorize, CUDAGUFuncVectorize 
 from numba.targets.registry import TargetRegistry
 
 
@@ -23,7 +25,8 @@ class _BaseVectorize(object):
 
 
 class Vectorize(_BaseVectorize):
-    target_registry = TargetRegistry({'cpu': UFuncBuilder})
+    target_registry = TargetRegistry({'cpu': UFuncBuilder,
+                                      'parallel': ParallelUFuncBuilder})
 
     def __new__(cls, func, **kws):
         identity = cls.get_identity(kws)
