@@ -40,9 +40,11 @@ class _Registry(object):
 # Registries of boxing / unboxing implementations
 _boxers = _Registry()
 _unboxers = _Registry()
+_reflectors = _Registry()
 
 box = _boxers.register
 unbox = _unboxers.register
+reflect = _reflectors.register
 
 class _BoxContext(namedtuple("_BoxContext",
                   ("context", "builder", "pyapi", "env_manager"))):
@@ -52,7 +54,7 @@ class _BoxContext(namedtuple("_BoxContext",
     __slots__ = ()
 
     def box(self, typ, val):
-        return self.pyapi.from_native_value(val, typ, self.env_manager)
+        return self.pyapi.from_native_value(typ, val, self.env_manager)
 
 
 class _UnboxContext(namedtuple("_UnboxContext",
@@ -63,7 +65,26 @@ class _UnboxContext(namedtuple("_UnboxContext",
     __slots__ = ()
 
     def unbox(self, typ, obj):
-        return self.pyapi.to_native_value(obj, typ)
+        return self.pyapi.to_native_value(typ, obj)
+
+
+class _ReflectContext(namedtuple("_ReflectContext",
+                      ("context", "builder", "pyapi", "env_manager",
+                       "is_error"))):
+    """
+    The facilities required by reflection implementations.
+    """
+    __slots__ = ()
+
+    # XXX the error bit is currently unused by consumers (e.g. PyCallWrapper)
+    def set_error(self):
+        self.builder.store(self.is_error, cgutils.true_bit)
+
+    def box(self, typ, val):
+        return self.pyapi.from_native_value(typ, val, self.env_manager)
+
+    def reflect(self, typ, val):
+        return self.pyapi.reflect_native_value(typ, val, self.env_manager)
 
 
 class NativeValue(object):
@@ -134,10 +155,12 @@ class PythonAPI(object):
 
         # Initialize types
         self.pyobj = self.context.get_argument_type(types.pyobject)
+        self.pyobjptr = self.pyobj.as_pointer()
         self.voidptr = Type.pointer(Type.int(8))
         self.long = Type.int(ctypes.sizeof(ctypes.c_long) * 8)
-        self.ulonglong = Type.int(ctypes.sizeof(ctypes.c_ulonglong) * 8)
-        self.longlong = self.ulonglong
+        self.ulong = self.long
+        self.longlong = Type.int(ctypes.sizeof(ctypes.c_ulonglong) * 8)
+        self.ulonglong = self.longlong
         self.double = Type.double()
         self.py_ssize_t = self.context.get_value_type(types.intp)
         self.cstring = Type.pointer(Type.int(8))
@@ -262,6 +285,45 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="PyErr_WriteUnraisable")
         return self.builder.call(fn, (obj,))
 
+    def err_fetch(self, pty, pval, ptb):
+        fnty = Type.function(Type.void(), [self.pyobjptr] * 3)
+        fn = self._get_function(fnty, name="PyErr_Fetch")
+        return self.builder.call(fn, (pty, pval, ptb))
+
+    def err_restore(self, ty, val, tb):
+        fnty = Type.function(Type.void(), [self.pyobj] * 3)
+        fn = self._get_function(fnty, name="PyErr_Restore")
+        return self.builder.call(fn, (ty, val, tb))
+
+    @contextlib.contextmanager
+    def err_push(self, keep_new=False):
+        """
+        Temporarily push the current error indicator while the code
+        block is executed.  If *keep_new* is True and the code block
+        raises a new error, the new error is kept, otherwise the old
+        error indicator is restored at the end of the block.
+        """
+        pty, pval, ptb = [cgutils.alloca_once(self.builder, self.pyobj)
+                          for i in range(3)]
+        self.err_fetch(pty, pval, ptb)
+        yield
+        ty = self.builder.load(pty)
+        val = self.builder.load(pval)
+        tb = self.builder.load(ptb)
+        if keep_new:
+            new_error = cgutils.is_not_null(self.builder, self.err_occurred())
+            with self.builder.if_else(new_error, likely=False) as (if_error, if_ok):
+                with if_error:
+                    # Code block raised an error, keep it
+                    self.decref(ty)
+                    self.decref(val)
+                    self.decref(tb)
+                with if_ok:
+                    # Restore previous error
+                    self.err_restore(ty, val, tb)
+        else:
+            self.err_restore(ty, val, tb)
+
     def get_c_object(self, name):
         """
         Get a Python object through its C-accessible *name*
@@ -270,7 +332,8 @@ class PythonAPI(object):
         """
         # A LLVM global variable is implicitly a pointer to the declared
         # type, so fix up by using pyobj.pointee.
-        return self.context.get_c_value(self.builder, self.pyobj.pointee, name)
+        return self.context.get_c_value(self.builder, self.pyobj.pointee, name,
+                                        dllimport=True)
 
     def raise_missing_global_error(self, name):
         msg = "global name '%s' is not defined" % name
@@ -431,6 +494,10 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name=func_name)
         return self.builder.call(fn, [ival])
 
+    def long_from_ulong(self, ival):
+        return self._long_from_native_int(ival, "PyLong_FromUnsignedLong",
+                                          self.long, signed=False)
+
     def long_from_ssize_t(self, ival):
         return self._long_from_native_int(ival, "PyLong_FromSsize_t",
                                           self.py_ssize_t, signed=True)
@@ -443,6 +510,30 @@ class PythonAPI(object):
         return self._long_from_native_int(ival, "PyLong_FromUnsignedLongLong",
                                           self.ulonglong, signed=False)
 
+    def long_from_signed_int(self, ival):
+        """
+        Return a Python integer from any native integer value.
+        """
+        bits = ival.type.width
+        if bits <= self.long.width:
+            return self.long_from_long(self.builder.sext(ival, self.long))
+        elif bits <= self.longlong.width:
+            return self.long_from_longlong(self.builder.sext(ival, self.longlong))
+        else:
+            raise OverflowError("integer too big (%d bits)" % (bits))
+
+    def long_from_unsigned_int(self, ival):
+        """
+        Same as long_from_signed_int, but for unsigned values.
+        """
+        bits = ival.type.width
+        if bits <= self.ulong.width:
+            return self.long_from_ulong(self.builder.zext(ival, self.ulong))
+        elif bits <= self.ulonglong.width:
+            return self.long_from_ulonglong(self.builder.zext(ival, self.ulonglong))
+        else:
+            raise OverflowError("integer too big (%d bits)" % (bits))
+
     def _get_number_operator(self, name):
         fnty = Type.function(self.pyobj, [self.pyobj, self.pyobj])
         fn = self._get_function(fnty, name="PyNumber_%s" % name)
@@ -549,6 +640,29 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="PyComplex_ImagAsDouble")
         return self.builder.call(fn, [cobj])
 
+    #
+    # Concrete slice API
+    #
+
+    def slice_as_ints(self, obj, defaults):
+        """
+        Read the members of a slice of integers.
+        Returns a (ok, start, stop, step) tuple where ok is a boolean and
+        the following members are pointer-sized ints.
+        """
+        defaults = [ir.Constant(self.py_ssize_t, v) for v in defaults]
+        pstart = cgutils.alloca_once_value(self.builder, defaults[0])
+        pstop = cgutils.alloca_once_value(self.builder, defaults[1])
+        pstep = cgutils.alloca_once_value(self.builder, defaults[2])
+        fnty = Type.function(Type.int(),
+                             [self.pyobj] + [self.py_ssize_t.as_pointer()] * 3)
+        fn = self._get_function(fnty, name="numba_unpack_slice")
+        res = self.builder.call(fn, (obj, pstart, pstop, pstep))
+        start = self.builder.load(pstart)
+        stop = self.builder.load(pstop)
+        step = self.builder.load(pstep)
+        return cgutils.is_null(self.builder, res), start, stop, step
+
     #
     # List and sequence APIs
     #
@@ -569,14 +683,24 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="PyList_New")
         return self.builder.call(fn, [szval])
 
-    def list_setitem(self, seq, idx, val):
+    def list_size(self, lst):
+        fnty = Type.function(self.py_ssize_t, [self.pyobj])
+        fn = self._get_function(fnty, name="PyList_Size")
+        return self.builder.call(fn, [lst])
+
+    def list_append(self, lst, val):
+        fnty = Type.function(Type.int(), [self.pyobj, self.pyobj])
+        fn = self._get_function(fnty, name="PyList_Append")
+        return self.builder.call(fn, [lst, val])
+
+    def list_setitem(self, lst, idx, val):
         """
         Warning: Steals reference to ``val``
         """
         fnty = Type.function(Type.int(), [self.pyobj, self.py_ssize_t,
                                           self.pyobj])
         fn = self._get_function(fnty, name="PyList_SetItem")
-        return self.builder.call(fn, [seq, idx, val])
+        return self.builder.call(fn, [lst, idx, val])
 
     def list_getitem(self, lst, idx):
         """
@@ -588,6 +712,30 @@ class PythonAPI(object):
             idx = self.context.get_constant(types.intp, idx)
         return self.builder.call(fn, [lst, idx])
 
+    def list_setslice(self, lst, start, stop, obj):
+        if obj is None:
+            obj = self.get_null_object()
+        fnty = Type.function(Type.int(), [self.pyobj, self.py_ssize_t,
+                                          self.py_ssize_t, self.pyobj])
+        fn = self._get_function(fnty, name="PyList_SetSlice")
+        return self.builder.call(fn, (lst, start, stop, obj))
+
+    def list_get_private_data(self, lst):
+        fnty = Type.function(self.voidptr, [self.pyobj])
+        fn = self._get_function(fnty, name="numba_get_list_private_data")
+        return self.builder.call(fn, (lst,))
+
+    def list_set_private_data(self, lst, ptr):
+        fnty = Type.function(Type.void(), [self.pyobj, self.voidptr])
+        fn = self._get_function(fnty, name="numba_set_list_private_data")
+        return self.builder.call(fn, (lst, ptr))
+
+    def list_reset_private_data(self, lst):
+        fnty = Type.function(Type.void(), [self.pyobj])
+        fn = self._get_function(fnty, name="numba_reset_list_private_data")
+        return self.builder.call(fn, (lst,))
+
+
     #
     # Concrete tuple API
     #
@@ -738,10 +886,10 @@ class PythonAPI(object):
             return self.builder.call(fn, (lhs, rhs, lopid))
         elif opstr == 'is':
             bitflag = self.builder.icmp(lc.ICMP_EQ, lhs, rhs)
-            return self.from_native_value(bitflag, types.boolean)
+            return self.from_native_value(types.boolean, bitflag)
         elif opstr == 'is not':
             bitflag = self.builder.icmp(lc.ICMP_NE, lhs, rhs)
-            return self.from_native_value(bitflag, types.boolean)
+            return self.from_native_value(types.boolean, bitflag)
         elif opstr == 'in':
             fnty = Type.function(Type.int(), [self.pyobj, self.pyobj])
             fn = self._get_function(fnty, name="PySequence_Contains")
@@ -893,13 +1041,12 @@ class PythonAPI(object):
         return self.builder.call(fn, [obj])
 
     def make_none(self):
-        obj = self.get_c_object("Py_None")
+        obj = self.borrow_none()
         self.incref(obj)
         return obj
 
     def borrow_none(self):
-        obj = self.get_c_object("Py_None")
-        return obj
+        return self.get_c_object("_Py_NoneStruct")
 
     def sys_write_stdout(self, fmt, *args):
         fnty = Type.function(Type.void(), [self.cstring], var_arg=True)
@@ -1014,6 +1161,24 @@ class PythonAPI(object):
         n = self.builder.extract_value(self.builder.load(structptr), 1)
         return self.builder.call(fn, (ptr, n))
 
+    def serialize_uncached(self, obj):
+        """
+        Same as serialize_object(), but don't create a global variable,
+        simply return a literal {i8* data, i32 length} structure.
+        """
+        # First make the array constant
+        data = pickle.dumps(obj, protocol=-1)
+        assert len(data) < 2**31
+        name = ".const.pickledata.%s" % (id(obj))
+        bdata = cgutils.make_bytearray(data)
+        arr = self.context.insert_unique_const(self.module, name, bdata)
+        # Then populate the structure constant
+        struct = ir.Constant.literal_struct([
+            arr.bitcast(self.voidptr),
+            ir.Constant(ir.IntType(32), arr.type.pointee.count),
+            ])
+        return struct
+
     def serialize_object(self, obj):
         """
         Serialize the given object in the bitcode, and return it
@@ -1023,16 +1188,7 @@ class PythonAPI(object):
         try:
             gv = self.module.__serialized[obj]
         except KeyError:
-            # First make the array constant
-            data = pickle.dumps(obj, protocol=-1)
-            name = ".const.pickledata.%s" % (id(obj))
-            bdata = cgutils.make_bytearray(data)
-            arr = self.context.insert_unique_const(self.module, name, bdata)
-            # Then populate the structure constant
-            struct = ir.Constant.literal_struct([
-                arr.bitcast(self.voidptr),
-                ir.Constant(ir.IntType(32), arr.type.pointee.count),
-                ])
+            struct = self.serialize_uncached(obj)
             name = ".const.picklebuf.%s" % (id(obj))
             gv = self.context.insert_unique_const(self.module, name, struct)
             # Make the id() (and hence the name) unique while populating the module.
@@ -1042,9 +1198,7 @@ class PythonAPI(object):
     def c_api_error(self):
         return cgutils.is_not_null(self.builder, self.err_occurred())
 
-    def to_native_value(self, obj, typ):
-        builder = self.builder
-
+    def to_native_value(self, typ, obj):
         impl = _unboxers.lookup(typ.__class__)
         if impl is None:
             raise NotImplementedError("cannot convert %s to native value" % (typ,))
@@ -1052,14 +1206,14 @@ class PythonAPI(object):
         c = _UnboxContext(self.context, self.builder, self)
         return impl(c, typ, obj)
 
-    def from_native_return(self, val, typ, env_manager):
+    def from_native_return(self, typ, val, env_manager):
         assert not isinstance(typ, types.Optional), "callconv should have " \
                                                     "prevented the return of " \
                                                     "optional value"
-        out = self.from_native_value(val, typ, env_manager)
+        out = self.from_native_value(typ, val, env_manager)
         return out
 
-    def from_native_value(self, val, typ, env_manager=None):
+    def from_native_value(self, typ, val, env_manager=None):
         impl = _boxers.lookup(typ.__class__)
         if impl is None:
             raise NotImplementedError("cannot convert native %s to Python object" % (typ,))
@@ -1067,6 +1221,22 @@ class PythonAPI(object):
         c = _BoxContext(self.context, self.builder, self, env_manager)
         return impl(c, typ, val)
 
+    def reflect_native_value(self, typ, val, env_manager=None):
+        """
+        Reflect the native value onto its Python original, if any.
+        An error bit (as an LLVM value) is returned.
+        """
+        impl = _reflectors.lookup(typ.__class__)
+        if impl is None:
+            # Reflection isn't needed for most types
+            return cgutils.false_bit
+
+        is_error = cgutils.alloca_once_value(self.builder, cgutils.false_bit)
+        c = _ReflectContext(self.context, self.builder, self, env_manager,
+                            is_error)
+        impl(c, typ, val)
+        return self.builder.load(c.is_error)
+
     def to_native_generator(self, obj, typ):
         """
         Extract the generator structure pointer from a generator *obj*
