@@ -1,11 +1,16 @@
-#include "_pymodule.h"
-#include "nrt.h"
+/*
+ * Definition of NRT functions for marshalling from / to Python objects.
+ * This module is included by _nrt_pythonmod.c and by pycc-compiled modules.
+ */
+
+#include "../_pymodule.h"
 
 #define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #include <numpy/ndarrayobject.h>
 #include <numpy/arrayscalars.h>
 
-#include "_arraystruct.h"
+#include "../_arraystruct.h"
+#include "nrt.h"
 
 /* For Numpy 1.6 */
 #ifndef NPY_ARRAY_BEHAVED
@@ -13,77 +18,12 @@
 #endif
 
 
-static
-PyObject*
-memsys_shutdown(PyObject *self, PyObject *args) {
-    NRT_MemSys_shutdown();
-    Py_RETURN_NONE;
-}
+/*
+ * Create a NRT MemInfo for data owned by a PyObject.
+ */
 
-static PyObject *
-memsys_use_cpython_allocator(PyObject *self, PyObject *args) {
-    NRT_MemSys_set_allocator(PyMem_RawMalloc,
-                             PyMem_RawRealloc,
-                             PyMem_RawFree);
-    Py_RETURN_NONE;
-}
-
-static
-PyObject*
-memsys_set_atomic_inc_dec(PyObject *self, PyObject *args) {
-    PyObject *addr_inc_obj, *addr_dec_obj;
-    void *addr_inc, *addr_dec;
-    if (!PyArg_ParseTuple(args, "OO", &addr_inc_obj, &addr_dec_obj)) {
-        return NULL;
-    }
-    addr_inc = PyLong_AsVoidPtr(addr_inc_obj);
-    if(PyErr_Occurred()) return NULL;
-    addr_dec = PyLong_AsVoidPtr(addr_dec_obj);
-    if(PyErr_Occurred()) return NULL;
-    NRT_MemSys_set_atomic_inc_dec(addr_inc, addr_dec);
-    Py_RETURN_NONE;
-}
-
-static
-PyObject*
-memsys_set_atomic_cas(PyObject *self, PyObject *args) {
-    PyObject *addr_cas_obj;
-    void *addr_cas;
-    if (!PyArg_ParseTuple(args, "O", &addr_cas_obj)) {
-        return NULL;
-    }
-    addr_cas = PyLong_AsVoidPtr(addr_cas_obj);
-    if(PyErr_Occurred()) return NULL;
-    NRT_MemSys_set_atomic_cas(addr_cas);
-    Py_RETURN_NONE;
-}
-
-static
-PyObject*
-memsys_get_stats_alloc(PyObject *self, PyObject *args) {
-    return PyLong_FromSize_t(NRT_MemSys_get_stats_alloc());
-}
-
-static
-PyObject*
-memsys_get_stats_free(PyObject *self, PyObject *args) {
-    return PyLong_FromSize_t(NRT_MemSys_get_stats_free());
-}
-
-static
-PyObject*
-memsys_get_stats_mi_alloc(PyObject *self, PyObject *args) {
-    return PyLong_FromSize_t(NRT_MemSys_get_stats_mi_alloc());
-}
-
-static
-PyObject*
-memsys_get_stats_mi_free(PyObject *self, PyObject *args) {
-    return PyLong_FromSize_t(NRT_MemSys_get_stats_mi_free());
-}
-
-static
-void pyobject_dtor(void *ptr, void* info) {
+static void
+pyobject_dtor(void *ptr, void* info) {
     PyGILState_STATE gstate;
     PyObject *ownerobj = info;
 
@@ -92,68 +32,20 @@ void pyobject_dtor(void *ptr, void* info) {
     PyGILState_Release(gstate);     /* release the GIL */
 }
 
-
-static
-MemInfo* meminfo_new_from_pyobject(void *data, PyObject *ownerobj) {
+static NRT_MemInfo *
+meminfo_new_from_pyobject(void *data, PyObject *ownerobj) {
     size_t dummy_size = 0;
     Py_INCREF(ownerobj);
     return NRT_MemInfo_new(data, dummy_size, pyobject_dtor, ownerobj);
 }
 
-
 /*
- * Create a new MemInfo with a owner PyObject
+ * A Python object wrapping a NRT meminfo.
  */
-static
-PyObject*
-meminfo_new(PyObject *self, PyObject *args) {
-    PyObject *addr_data_obj;
-    void *addr_data;
-    PyObject *ownerobj;
-    MemInfo *mi;
-    if (!PyArg_ParseTuple(args, "OO", &addr_data_obj, &ownerobj)) {
-        return NULL;
-    }
-    addr_data = PyLong_AsVoidPtr(addr_data_obj);
-    if(PyErr_Occurred()) return NULL;
-    mi = meminfo_new_from_pyobject(addr_data, ownerobj);
-    return PyLong_FromVoidPtr(mi);
-}
-
-/*
- * Create a new MemInfo with a new NRT allocation
- */
-static
-PyObject*
-meminfo_alloc(PyObject *self, PyObject *args) {
-    MemInfo *mi;
-    Py_ssize_t size;
-    if (!PyArg_ParseTuple(args, "n", &size)) {
-        return NULL;
-    }
-    mi = NRT_MemInfo_alloc(size);
-    return PyLong_FromVoidPtr(mi);
-}
-
-/*
- * Like meminfo_alloc but set memory to zero after allocation and before
- * deallocation.
- */
-static
-PyObject*
-meminfo_alloc_safe(PyObject *self, PyObject *args) {
-    MemInfo *mi;
-    Py_ssize_t size;
-    if (!PyArg_ParseTuple(args, "n", &size)) {
-        return NULL;
-    }
-    mi = NRT_MemInfo_alloc_safe(size);
-    return PyLong_FromVoidPtr(mi);
-}
 
 typedef struct {
     PyObject_HEAD
-    MemInfo *meminfo;
+    NRT_MemInfo *meminfo;
 } MemInfoObject;
 
 static
@@ -166,29 +58,18 @@ int MemInfo_init(MemInfoObject *self, PyObject *args, PyObject *kwds) {
     }
     raw_ptr = PyLong_AsVoidPtr(raw_ptr_obj);
     if(PyErr_Occurred()) return -1;
-    self->meminfo = (MemInfo*)raw_ptr;
+    self->meminfo = (NRT_MemInfo *)raw_ptr;
     assert (NRT_MemInfo_refcount(self->meminfo) > 0 && "0 refcount");
     return 0;
 }
 
-int MemInfo_getbuffer(PyObject *exporter, Py_buffer *view, int flags) {
-    Py_ssize_t len;
-    void *buf;
-    int readonly = 0;
 
-    MemInfoObject *miobj = (MemInfoObject*)exporter;
-    MemInfo *mi = miobj->meminfo;
-
-    buf = NRT_MemInfo_data(mi);
-    len = NRT_MemInfo_size(mi);
-    return PyBuffer_FillInfo(view, exporter, buf, len, readonly, flags);
-}
-
-Py_ssize_t MemInfo_rdwrbufferproc(PyObject *self, Py_ssize_t segment,
-                                  void **ptrptr)
+#if PY_MAJOR_VERSION < 3
+static Py_ssize_t
+MemInfo_rdwrbufferproc(PyObject *self, Py_ssize_t segment, void **ptrptr)
 {
     MemInfoObject *mio = (MemInfoObject *)self;
-    MemInfo *mi = mio->meminfo;
+    NRT_MemInfo *mi = mio->meminfo;
     if (segment != 0) {
         PyErr_SetString(PyExc_TypeError, "MemInfo only has 1 segment");
         return -1;
@@ -197,16 +78,34 @@ Py_ssize_t MemInfo_rdwrbufferproc(PyObject *self, Py_ssize_t segment,
     return NRT_MemInfo_size(mi);
 }
 
-Py_ssize_t MemInfo_segcountproc(PyObject *self, Py_ssize_t *lenp) {
+static Py_ssize_t
+MemInfo_segcountproc(PyObject *self, Py_ssize_t *lenp) {
     MemInfoObject *mio = (MemInfoObject *)self;
-    MemInfo *mi = mio->meminfo;
+    NRT_MemInfo *mi = mio->meminfo;
     if (lenp) {
         *lenp = NRT_MemInfo_size(mi);
     }
     return 1;
 }
 
-#if (PY_MAJOR_VERSION < 3)
+#else /* PY_MAJOR_VERSION < 3 */
+
+static int
+MemInfo_getbuffer(PyObject *exporter, Py_buffer *view, int flags) {
+    Py_ssize_t len;
+    void *buf;
+    int readonly = 0;
+
+    MemInfoObject *miobj = (MemInfoObject*)exporter;
+    NRT_MemInfo *mi = miobj->meminfo;
+
+    buf = NRT_MemInfo_data(mi);
+    len = NRT_MemInfo_size(mi);
+    return PyBuffer_FillInfo(view, exporter, buf, len, readonly, flags);
+}
+#endif /* PY_MAJOR_VERSION < 3 */
+
+#if PY_MAJOR_VERSION < 3
 static PyBufferProcs MemInfo_bufferProcs = {MemInfo_rdwrbufferproc,
                                             MemInfo_rdwrbufferproc,
                                             MemInfo_segcountproc,
@@ -324,11 +223,12 @@ static PyTypeObject MemInfoType = {
 };
 
 
-/****** Array adaptor code ******/
+/*
+ * Array adaptor code
+ */
 
-
-static
-int NRT_adapt_ndarray_from_python(PyObject *obj, arystruct_t* arystruct) {
+NUMBA_EXPORT_FUNC(int)
+NRT_adapt_ndarray_from_python(PyObject *obj, arystruct_t* arystruct) {
     PyArrayObject *ndary;
     int i, ndim;
     npy_intp *p;
@@ -397,7 +297,7 @@ RETURN_ARRAY_COPY:
     return NULL;
 }
 
-static PyObject *
+NUMBA_EXPORT_FUNC(PyObject *)
 NRT_adapt_ndarray_to_python(arystruct_t* arystruct, int ndim,
                             int writeable, PyArray_Descr *descr)
 {
@@ -483,7 +383,7 @@ NRT_adapt_ndarray_to_python(arystruct_t* arystruct, int ndim,
     return (PyObject *) array;
 }
 
-static void
+NUMBA_EXPORT_FUNC(void)
 NRT_adapt_buffer_from_python(Py_buffer *buf, arystruct_t *arystruct)
 {
     int i;
@@ -505,99 +405,16 @@ NRT_adapt_buffer_from_python(Py_buffer *buf, arystruct_t *arystruct)
     for (i = 0; i < buf->ndim; i++, p++) {
         *p = buf->strides[i];
     }
-
-
 }
 
-static void
-NRT_incref(MemInfo* mi) {
-    if (mi) {
-        NRT_MemInfo_acquire(mi);
-    }
-}
 
-static void
-NRT_decref(MemInfo* mi) {
-    if (mi) {
-        NRT_MemInfo_release(mi);
-    }
-}
+/* Initialization subroutines for modules including this source file */
 
-static PyMethodDef ext_methods[] = {
-#define declmethod(func) { #func , ( PyCFunction )func , METH_VARARGS , NULL }
-#define declmethod_noargs(func) { #func , ( PyCFunction )func , METH_NOARGS, NULL }
-    declmethod_noargs(memsys_use_cpython_allocator),
-    declmethod_noargs(memsys_shutdown),
-    declmethod(memsys_set_atomic_inc_dec),
-    declmethod(memsys_set_atomic_cas),
-    declmethod_noargs(memsys_get_stats_alloc),
-    declmethod_noargs(memsys_get_stats_free),
-    declmethod_noargs(memsys_get_stats_mi_alloc),
-    declmethod_noargs(memsys_get_stats_mi_free),
-    declmethod(meminfo_new),
-    declmethod(meminfo_alloc),
-    declmethod(meminfo_alloc_safe),
-    { NULL },
-#undef declmethod
-};
-
-
-
-static PyObject *
-build_c_helpers_dict(void)
+static int
+init_nrt_python_module(PyObject *module)
 {
-    PyObject *dct = PyDict_New();
-    if (dct == NULL)
-        goto error;
-
-#define _declpointer(name, value) do {                 \
-    PyObject *o = PyLong_FromVoidPtr(value);           \
-    if (o == NULL) goto error;                         \
-    if (PyDict_SetItemString(dct, name, o)) {          \
-        Py_DECREF(o);                                  \
-        goto error;                                    \
-    }                                                  \
-    Py_DECREF(o);                                      \
-} while (0)
-
-#define declmethod(func) _declpointer(#func, &NRT_##func)
-
-declmethod(adapt_ndarray_from_python);
-declmethod(adapt_ndarray_to_python);
-declmethod(adapt_buffer_from_python);
-declmethod(incref);
-declmethod(decref);
-declmethod(MemInfo_alloc);
-declmethod(MemInfo_alloc_safe);
-declmethod(MemInfo_alloc_aligned);
-declmethod(MemInfo_alloc_safe_aligned);
-declmethod(MemInfo_call_dtor);
-declmethod(MemInfo_varsize_alloc);
-declmethod(MemInfo_varsize_realloc);
-
-
-#undef declmethod
-    return dct;
-error:
-    Py_XDECREF(dct);
-    return NULL;
-}
-
-MOD_INIT(_nrt_python) {
-    PyObject *m;
-    MOD_DEF(m, "_nrt_python", "No docs", ext_methods)
-    if (m == NULL)
-        return MOD_ERROR_VAL;
-    import_array();
-    NRT_MemSys_init();
     MemInfoType.tp_new = PyType_GenericNew;
     if (PyType_Ready(&MemInfoType))
-        return MOD_ERROR_VAL;
-
-    Py_INCREF(&MemInfoType);
-    PyModule_AddObject(m, "_MemInfo", (PyObject *) (&MemInfoType));
-
-    PyModule_AddObject(m, "c_helpers", build_c_helpers_dict());
-
-    return MOD_SUCCESS_VAL(m);
-}
+        return -1;
+    return 0;
+}
\ No newline at end of file
