@@ -12,8 +12,8 @@ from .base import BaseContext, PYOBJECT
 from numba import utils, cgutils, types
 from numba.utils import cached_property
 from numba.targets import (
-    callconv, codegen, externals, intrinsics, listobj, cmathimpl, mathimpl,
-    npyimpl, operatorimpl, printimpl, randomimpl)
+    callconv, cffiimpl, codegen, externals, intrinsics, listobj, cmathimpl,
+    mathimpl, npyimpl, operatorimpl, printimpl, randomimpl)
 from .options import TargetOptions
 from numba.runtime import rtsys
 
@@ -40,21 +40,21 @@ class CPUContext(BaseContext):
 
     def init(self):
         self.is32bit = (utils.MACHINE_BITS == 32)
+        self._internal_codegen = codegen.JITCPUCodegen("numba.exec")
 
         # Map external C functions.
-        externals.c_math_functions.install()
-        externals.c_numpy_functions.install()
+        externals.c_math_functions.install(self)
+        externals.c_numpy_functions.install(self)
 
         # Add target specific implementations
         self.install_registry(cmathimpl.registry)
+        self.install_registry(cffiimpl.registry)
         self.install_registry(mathimpl.registry)
         self.install_registry(npyimpl.registry)
         self.install_registry(operatorimpl.registry)
         self.install_registry(printimpl.registry)
         self.install_registry(randomimpl.registry)
 
-        self._internal_codegen = codegen.JITCPUCodegen("numba.exec")
-
         # Initialize NRT runtime
         rtsys.initialize(self)
 
@@ -62,10 +62,11 @@ class CPUContext(BaseContext):
     def target_data(self):
         return self._internal_codegen.target_data
 
-    def aot_codegen(self, name):
-        return codegen.AOTCPUCodegen(name)
+    def with_aot_codegen(self, name):
+        return self.subtarget(_internal_codegen=codegen.AOTCPUCodegen(name),
+                              aot_mode=True)
 
-    def jit_codegen(self):
+    def codegen(self):
         return self._internal_codegen
 
     @cached_property
