@@ -10,11 +10,10 @@ from llvmlite import ir
 from numba import types, cgutils, typing
 from numba.targets.imputils import (builtin, builtin_attr, implement,
                                     impl_attribute, impl_attribute_generic,
-                                    iternext_impl, struct_factory,
-                                    impl_ret_borrowed, impl_ret_new_ref,
-                                    impl_ret_untracked)
+                                    iternext_impl, impl_ret_borrowed,
+                                    impl_ret_new_ref, impl_ret_untracked)
 from numba.utils import cached_property
-from . import slicing
+from . import quicksort, slicing
 
 
 def make_list_cls(list_type):
@@ -30,7 +29,10 @@ def make_payload_cls(list_type):
     Return the Structure representation of the given *list_type*'s payload
     (an instance of types.List).
     """
-    return cgutils.create_struct_proxy(types.ListPayload(list_type))
+    # Note the payload is stored durably in memory, so we consider it
+    # data and not value.
+    return cgutils.create_struct_proxy(types.ListPayload(list_type),
+                                       kind='data')
 
 
 def get_list_payload(context, builder, list_type, value):
@@ -62,6 +64,10 @@ class _ListPayloadMixin(object):
     def size(self, value):
         self._payload.size = value
 
+    @property
+    def dirty(self):
+        return self._payload.dirty
+
     @property
     def data(self):
         return self._payload._get_ptr_by_name('data')
@@ -75,16 +81,9 @@ class _ListPayloadMixin(object):
 
     def getitem(self, idx):
         ptr = self._gep(idx)
-        return self._builder.load(ptr)
+        data_item = self._builder.load(ptr)
+        return self._datamodel.from_data(self._builder, data_item)
 
-    def setitem(self, idx, val):
-        ptr = self._gep(idx)
-        self._builder.store(val, ptr)
-
-    def inititem(self, idx, val):
-        ptr = self._gep(idx)
-        self._builder.store(val, ptr)
-    
     def fix_index(self, idx):
         """
         Fix negative indices by adding the size to them.  Positive
@@ -147,12 +146,25 @@ class ListInstance(_ListPayloadMixin):
         self._ty = list_type
         self._list = make_list_cls(list_type)(context, builder, list_val)
         self._itemsize = get_itemsize(context, list_type)
+        self._datamodel = context.data_model_manager[list_type.dtype]
+
+    @property
+    def dtype(self):
+        return self._ty.dtype
 
     @property
     def _payload(self):
         # This cannot be cached as it can be reallocated
         return get_list_payload(self._context, self._builder, self._ty, self._list)
 
+    @property
+    def parent(self):
+        return self._list.parent
+
+    @parent.setter
+    def parent(self, value):
+        self._list.parent = value
+
     @property
     def value(self):
         return self._list._getvalue()
@@ -161,8 +173,29 @@ class ListInstance(_ListPayloadMixin):
     def meminfo(self):
         return self._list.meminfo
 
+    def set_dirty(self, val):
+        if self._ty.reflected:
+            self._payload.dirty = cgutils.true_bit if val else cgutils.false_bit
+
+    def setitem(self, idx, val):
+        ptr = self._gep(idx)
+        data_item = self._datamodel.as_data(self._builder, val)
+        self._builder.store(data_item, ptr)
+        self.set_dirty(True)
+
+    def inititem(self, idx, val):
+        ptr = self._gep(idx)
+        data_item = self._datamodel.as_data(self._builder, val)
+        self._builder.store(data_item, ptr)
+
     @classmethod
-    def allocate(cls, context, builder, list_type, nitems):
+    def allocate_ex(cls, context, builder, list_type, nitems):
+        """
+        Allocate a ListInstance with its storage.
+        Return a (ok, instance) tuple where *ok* is a LLVM boolean and
+        *instance* is a ListInstance object (the object's contents are
+        only valid when *ok* is true).
+        """
         intp_t = context.get_value_type(types.intp)
 
         if isinstance(nitems, int):
@@ -172,23 +205,58 @@ class ListInstance(_ListPayloadMixin):
         payload_size = context.get_abi_sizeof(payload_type)
 
         itemsize = get_itemsize(context, list_type)
-        
+
+        ok = cgutils.alloca_once_value(builder, cgutils.true_bit)
+        self = cls(context, builder, list_type, None)
+
         # Total allocation size = <payload header size> + nitems * itemsize
         allocsize, ovf = cgutils.muladd_with_overflow(builder, nitems,
                                                       ir.Constant(intp_t, itemsize),
                                                       ir.Constant(intp_t, payload_size))
         with builder.if_then(ovf, likely=False):
+            builder.store(cgutils.false_bit, ok)
+
+        with builder.if_then(builder.load(ok), likely=True):
+            meminfo = context.nrt_meminfo_varsize_alloc(builder, size=allocsize)
+            with builder.if_else(cgutils.is_null(builder, meminfo),
+                                 likely=False) as (if_error, if_ok):
+                with if_error:
+                    builder.store(cgutils.false_bit, ok)
+                with if_ok:
+                    self._list.meminfo = meminfo
+                    self._list.parent = context.get_constant_null(types.pyobject)
+                    self._payload.allocated = nitems
+                    self._payload.size = ir.Constant(intp_t, 0)  # for safety
+                    self._payload.dirty = cgutils.false_bit
+
+        return builder.load(ok), self
+
+    @classmethod
+    def allocate(cls, context, builder, list_type, nitems):
+        """
+        Allocate a ListInstance with its storage.  Same as allocate_ex(),
+        but return an initialized *instance*.  If allocation failed,
+        control is transferred to the caller using the target's current
+        call convention.
+        """
+        ok, self = cls.allocate_ex(context, builder, list_type, nitems)
+        with builder.if_then(builder.not_(ok), likely=False):
             context.call_conv.return_user_exc(builder, MemoryError,
                                               ("cannot allocate list",))
+        return self
 
-        meminfo = context.nrt_meminfo_varsize_alloc(builder, size=allocsize)
-        cgutils.guard_memory_error(context, builder, meminfo,
-                                   "cannot allocate list")
-
+    @classmethod
+    def from_meminfo(cls, context, builder, list_type, meminfo):
+        """
+        Allocate a new list instance pointing to an existing payload
+        (a meminfo pointer).
+        Note the parent field has to be filled by the caller.
+        """
         self = cls(context, builder, list_type, None)
         self._list.meminfo = meminfo
-        self._payload.allocated = nitems
-        self._payload.size = ir.Constant(intp_t, 0)  # for safety
+        self._list.parent = context.get_constant_null(types.pyobject)
+        context.nrt_incref(builder, list_type, self.value)
+        # Payload is part of the meminfo, no need to touch it
         return self
 
     def resize(self, new_size):
@@ -243,6 +311,7 @@ class ListInstance(_ListPayloadMixin):
             _payload_realloc(new_allocated)
 
         self._payload.size = new_size
+        self.set_dirty(True)
 
     def move(self, dest_idx, src_idx, count):
         """
@@ -252,6 +321,7 @@ class ListInstance(_ListPayloadMixin):
         src_ptr = self._gep(src_idx)
         cgutils.memmove(self._builder, dest_ptr, src_ptr,
                         count, itemsize=self._itemsize)
+        self.set_dirty(True)
 
 
 class ListIterInstance(_ListPayloadMixin):
@@ -261,6 +331,7 @@ class ListIterInstance(_ListPayloadMixin):
         self._builder = builder
         self._ty = iter_type
         self._iter = make_listiter_cls(iter_type)(context, builder, iter_val)
+        self._datamodel = context.data_model_manager[iter_type.yield_type]
 
     @classmethod
     def from_list(cls, context, builder, iter_type, list_val):
@@ -329,7 +400,6 @@ def list_len(context, builder, sig, args):
     return inst.size
 
 
-@struct_factory(types.ListIter)
 def make_listiter_cls(iterator_type):
     """
     Return the Structure representation of the given *iterator_type* (an
@@ -532,9 +602,11 @@ def list_add(context, builder, sig, args):
 
     with cgutils.for_range(builder, a_size) as loop:
         value = a.getitem(loop.index)
+        value = context.cast(builder, value, a.dtype, dest.dtype)
         dest.setitem(loop.index, value)
     with cgutils.for_range(builder, b_size) as loop:
         value = b.getitem(loop.index)
+        value = context.cast(builder, value, b.dtype, dest.dtype)
         dest.setitem(builder.add(loop.index, a_size), value)
 
     return impl_ret_new_ref(context, builder, sig.return_type, dest.value)
@@ -542,7 +614,7 @@ def list_add(context, builder, sig, args):
 @builtin
 @implement("+=", types.Kind(types.List), types.Kind(types.List))
 def list_add_inplace(context, builder, sig, args):
-    assert sig.args[0].dtype == sig.args[1].dtype
+    assert sig.args[0].dtype == sig.return_type.dtype
     dest = _list_extend_list(context, builder, sig, args)
 
     return impl_ret_borrowed(context, builder, sig.return_type, dest.value)
@@ -743,6 +815,7 @@ def _list_extend_list(context, builder, sig, args):
 
     with cgutils.for_range(builder, src_size) as loop:
         value = src.getitem(loop.index)
+        value = context.cast(builder, value, src.dtype, dest.dtype)
         dest.setitem(builder.add(loop.index, dest_size), value)
 
     return dest
@@ -750,8 +823,8 @@ def _list_extend_list(context, builder, sig, args):
 @builtin
 @implement("list.extend", types.Kind(types.List), types.Kind(types.IterableType))
 def list_extend(context, builder, sig, args):
-    if isinstance(sig.args[1], types.List) and sig.args[0].dtype == sig.args[1].dtype:
-        # Specialize for same-type list operands, for speed
+    if isinstance(sig.args[1], types.List):
+        # Specialize for list operands, for speed.
         _list_extend_list(context, builder, sig, args)
         return context.get_dummy_value()
 
@@ -892,3 +965,60 @@ def list_reverse(context, builder, sig, args):
             lst[a], lst[b] = lst[b], lst[a]
 
     return context.compile_internal(builder, list_reverse_impl, sig, args)
+
+
+# -----------------------------------------------------------------------------
+# Sorting
+
+_sorting_init = False
+
+def load_sorts():
+    """
+    Load quicksort lazily, to avoid circular imports accross the jit() global.
+    """
+    g = globals()
+    if g['_sorting_init']:
+        return
+
+    def gt(a, b):
+        return a > b
+
+    default_sort = quicksort.make_jit_quicksort()
+    reversed_sort = quicksort.make_jit_quicksort(lt=gt)
+    g['run_default_sort'] = default_sort.run_quicksort
+    g['run_reversed_sort'] = reversed_sort.run_quicksort
+    g['_sorting_init'] = True
+
+
+@builtin
+@implement("list.sort", types.Kind(types.List))
+@implement("list.sort", types.Kind(types.List), types.Kind(types.Boolean))
+def list_sort(context, builder, sig, args):
+    load_sorts()
+
+    if len(args) == 1:
+        sig = typing.signature(sig.return_type, *sig.args + (types.boolean,))
+        args = tuple(args) + (cgutils.false_bit,)
+
+    def list_sort_impl(lst, reverse):
+        if reverse:
+            return run_reversed_sort(lst)
+        else:
+            return run_default_sort(lst)
+
+    return context.compile_internal(builder, list_sort_impl, sig, args)
+
+@builtin
+@implement(sorted, types.Kind(types.IterableType))
+@implement(sorted, types.Kind(types.IterableType), types.Kind(types.Boolean))
+def sorted_impl(context, builder, sig, args):
+    if len(args) == 1:
+        sig = typing.signature(sig.return_type, *sig.args + (types.boolean,))
+        args = tuple(args) + (cgutils.false_bit,)
+
+    def sorted_impl(it, reverse):
+        lst = list(it)
+        lst.sort(reverse=reverse)
+        return lst
+
+    return context.compile_internal(builder, sorted_impl, sig, args)
