@@ -4,12 +4,15 @@ Assorted utilities for use in tests.
 
 import cmath
 import contextlib
+import errno
 import math
+import os
 import sys
+import tempfile
 
 import numpy as np
 
-from numba import config, errors, typing, utils
+from numba import config, errors, typing, utils, numpy_support
 from numba.compiler import compile_extra, compile_isolated, Flags, DEFAULT_FLAGS
 from numba.targets import cpu
 import numba.unittest_support as unittest
@@ -25,7 +28,7 @@ force_pyobj_flags.set("force_pyobject")
 no_pyobj_flags = Flags()
 
 
-is_on_numpy_16 = np.__version__.startswith("1.6.")
+is_on_numpy_16 = numpy_support.version == (1, 6)
 skip_on_numpy_16 = unittest.skipIf(is_on_numpy_16,
                                    "test requires Numpy 1.7 or later")
 
@@ -90,6 +93,21 @@ class TestCase(unittest.TestCase):
         with self.assertRaises(_accepted_errors) as cm:
             yield cm
 
+    @contextlib.contextmanager
+    def assertRefCount(self, *objects):
+        """
+        A context manager that asserts the given objects have the
+        same reference counts before and after executing the
+        enclosed blocks.
+        """
+        old_refcounts = [sys.getrefcount(x) for x in objects]
+        yield
+        new_refcounts = [sys.getrefcount(x) for x in objects]
+        for old, new, obj in zip(old_refcounts, new_refcounts, objects):
+            if old != new:
+                self.fail("Refcount changed from %d to %d for object: %r"
+                          % (old, new, obj))
+
     _exact_typesets = [(bool, np.bool_), utils.INT_TYPES, (str,), (np.integer,), (utils.text_type), ]
     _approx_typesets = [(float,), (complex,), (np.inexact)]
     _sequence_typesets = [(tuple, list)]
@@ -144,6 +162,15 @@ class TestCase(unittest.TestCase):
                 for (stride, shape) in zip(arr.strides, arr.shape)
                 if shape > 1]
 
+    def assertStridesEqual(self, first, second):
+        """
+        Test that two arrays have the same shape and strides.
+        """
+        self.assertEqual(first.shape, second.shape, "shapes differ")
+        self.assertEqual(first.itemsize, second.itemsize, "itemsizes differ")
+        self.assertEqual(self._fix_strides(first), self._fix_strides(second),
+                         "strides differ")
+
     def assertPreciseEqual(self, first, second, prec='exact', ulps=1,
                            msg=None):
         """
@@ -337,6 +364,20 @@ def tweak_code(func, codestring=None, consts=None):
                       co.co_lnotab)
     func.__code__ = new_code
 
+def static_temp_directory(dirname):
+    """
+    Create a directory in the temp dir with a given name. Statically-named
+    temp dirs created using this function are needed because we can't delete a
+    DLL under Windows (this is a bit fragile if stale files can influence the
+    result of future test runs).
+    """
+    tmpdir = os.path.join(tempfile.gettempdir(), dirname)
+    try:
+        os.mkdir(tmpdir)
+    except OSError as e:
+        if e.errno != errno.EEXIST:
+            raise
+    return tmpdir
 
 # From CPython
 
