@@ -13,14 +13,21 @@ from numba.utils import IS_PY3
 def get_a(ary, i):
     return ary[i].a
 
-
 def get_b(ary, i):
     return ary[i].b
 
-
 def get_c(ary, i):
     return ary[i].c
 
+def get_a_subarray(ary, i):
+    return ary.a[i]
+
+def get_b_subarray(ary, i):
+    return ary.b[i]
+
+def get_c_subarray(ary, i):
+    return ary.c[i]
+
 
 def get_two_arrays_a(ary1, ary2, i):
     return ary1[i].a + ary2[i].a
@@ -39,14 +46,20 @@ def get_two_arrays_distinct(ary1, ary2, i):
 def set_a(ary, i, v):
     ary[i].a = v
 
-
 def set_b(ary, i, v):
     ary[i].b = v
 
-
 def set_c(ary, i, v):
     ary[i].c = v
 
+def set_a_subarray(ary, i, v):
+    ary.a[i] = v
+
+def set_b_subarray(ary, i, v):
+    ary.b[i] = v
+
+def set_c_subarray(ary, i, v):
+    ary.c[i] = v
 
 def set_record(ary, i, j):
     ary[i] = ary[j]
@@ -163,12 +176,12 @@ def get_charseq_tuple(ary, i):
 
 
 recordtype = np.dtype([('a', np.float64),
-                       ('b', np.int32),
+                       ('b', np.int16),
                        ('c', np.complex64),
                        ('d', (np.str, 5))])
 
 recordtype2 = np.dtype([('e', np.int32),
-                        ('f', np.float64)])
+                        ('f', np.float64)], align=True)
 
 recordtype3 = np.dtype([('first', np.float32),
                         ('second', np.float64)])
@@ -235,7 +248,7 @@ class TestRecordDtype(unittest.TestCase):
     def test_from_dtype(self):
         rec = numpy_support.from_dtype(recordtype)
         self.assertEqual(rec.typeof('a'), types.float64)
-        self.assertEqual(rec.typeof('b'), types.int32)
+        self.assertEqual(rec.typeof('b'), types.int16)
         self.assertEqual(rec.typeof('c'), types.complex64)
         if IS_PY3:
             self.assertEqual(rec.typeof('d'), types.UnicodeCharSeq(5))
@@ -255,12 +268,15 @@ class TestRecordDtype(unittest.TestCase):
 
     def test_get_a(self):
         self._test_get_equal(get_a)
+        self._test_get_equal(get_a_subarray)
 
     def test_get_b(self):
         self._test_get_equal(get_b)
+        self._test_get_equal(get_b_subarray)
 
     def test_get_c(self):
         self._test_get_equal(get_c)
+        self._test_get_equal(get_c_subarray)
 
     def _test_get_two_equal(self, pyfunc):
         '''
@@ -309,19 +325,28 @@ class TestRecordDtype(unittest.TestCase):
             self.assertTrue(np.all(expect == got))
 
     def test_set_a(self):
-        self._test_set_equal(set_a, 3.1415, types.float64)
-        # Test again to check if coercion works
-        self._test_set_equal(set_a, 3., types.float32)
+        def check(pyfunc):
+            self._test_set_equal(pyfunc, 3.1415, types.float64)
+            # Test again to check if coercion works
+            self._test_set_equal(pyfunc, 3., types.float32)
+        check(set_a)
+        check(set_a_subarray)
 
     def test_set_b(self):
-        self._test_set_equal(set_b, 123, types.int32)
-        # Test again to check if coercion works
-        self._test_set_equal(set_b, 123, types.float64)
+        def check(pyfunc):
+            self._test_set_equal(pyfunc, 123, types.int32)
+            # Test again to check if coercion works
+            self._test_set_equal(pyfunc, 123, types.float64)
+        check(set_b)
+        check(set_b_subarray)
 
     def test_set_c(self):
-        self._test_set_equal(set_c, 43j, types.complex64)
-        # Test again to check if coercion works
-        self._test_set_equal(set_c, 43j, types.complex128)
+        def check(pyfunc):
+            self._test_set_equal(pyfunc, 43j, types.complex64)
+            # Test again to check if coercion works
+            self._test_set_equal(pyfunc, 43j, types.complex128)
+        check(set_c)
+        check(set_c_subarray)
 
     def test_set_record(self):
         pyfunc = set_record
@@ -349,8 +374,8 @@ class TestRecordDtype(unittest.TestCase):
         npval = self.refsample1d.copy()[0]
         nbval = self.nbsample1d.copy()[0]
         attrs = 'abc'
-        valtypes = types.float64, types.int32, types.complex64
-        values = 1.23, 123432, 132j
+        valtypes = types.float64, types.int16, types.complex64
+        values = 1.23, 12345, 123+456j
         old_refcnt = sys.getrefcount(nbval)
 
         for attr, valtyp, val in zip(attrs, valtypes, values):
@@ -372,18 +397,27 @@ class TestRecordDtype(unittest.TestCase):
             cfunc = self.get_cfunc(pyfunc, argtypes)
 
             got = cfunc(*args)
-            self.assertEqual(expected, got)
-            self.assertNotEqual(nbval['a'], got)
+            try:
+                self.assertEqual(expected, got)
+            except AssertionError as e:
+                # On ARM, a LLVM misoptimization can produce buggy code,
+                # see https://llvm.org/bugs/show_bug.cgi?id=24669
+                import llvmlite.binding as ll
+                if attr != 'c':
+                    raise
+                if ll.get_default_triple() != 'armv7l-unknown-linux-gnueabihf':
+                    raise
+                self.assertEqual(val, got)
+            else:
+                self.assertEqual(nbval[attr], val)
             del got, expected, args
 
         # Check for potential leaks (issue #441)
         self.assertEqual(sys.getrefcount(nbval), old_refcnt)
 
-
     def test_record_args(self):
         self._test_record_args(False)
 
-
     def test_record_args_reverse(self):
         self._test_record_args(True)
 
@@ -508,11 +542,10 @@ class TestRecordDtype(unittest.TestCase):
         attrs = 'abc'
         indices = [0, 1, 2]
         for index, attr in zip(indices, attrs):
-            npary = self.refsample1d.copy()
             nbary = self.nbsample1d.copy()
             old_refcnt = sys.getrefcount(nbary)
-            res = cfunc(npary, index)
-            self.assertEqual(npary[index], res)
+            res = cfunc(nbary, index)
+            self.assertEqual(nbary[index], res)
             # Prove that this is a by-value copy
             setattr(res, attr, 0)
             self.assertNotEqual(nbary[index], res)
