@@ -27,9 +27,12 @@ recordtype = np.dtype([('a', np.float64),
                        ('c', np.complex64),
                        ('d', (np.str, 5))])
 
-recordtype2 = np.dtype([('e', np.int32),
+recordtype2 = np.dtype([('e', np.int8),
                         ('f', np.float64)])
 
+recordtype3 = np.dtype([('e', np.int8),
+                        ('f', np.float64)], align=True)
+
 Point = namedtuple('Point', ('x', 'y'))
 
 Rect = namedtuple('Rect', ('width', 'height'))
@@ -61,6 +64,8 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertEqual(typeof(-1), types.intp)
         self.assertEqual(typeof(2**40), types.int64)
         self.assertEqual(typeof(2**63), types.uint64)
+        self.assertEqual(typeof(2**63 - 1), types.int64)
+        self.assertEqual(typeof(-2**63), types.int64)
 
     def test_datetime_values(self):
         """
@@ -78,40 +83,53 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         """
         Test special.typeof() with ndarray values.
         """
-        def check(arr, ndim, layout, mutable):
+        def check(arr, ndim, layout, mutable, aligned):
             ty = typeof(arr)
             self.assertIsInstance(ty, types.Array)
             self.assertEqual(ty.ndim, ndim)
             self.assertEqual(ty.layout, layout)
             self.assertEqual(ty.mutable, mutable)
+            self.assertEqual(ty.aligned, aligned)
 
         a1 = np.arange(10)
-        check(a1, 1, 'C', True)
+        check(a1, 1, 'C', True, True)
         a2 = np.arange(10).reshape(2, 5)
-        check(a2, 2, 'C', True)
-        check(a2.T, 2, 'F', True)
+        check(a2, 2, 'C', True, True)
+        check(a2.T, 2, 'F', True, True)
         a3 = (np.arange(60))[::2].reshape((2, 5, 3))
-        check(a3, 3, 'A', True)
+        check(a3, 3, 'A', True, True)
         a4 = np.arange(1).reshape(())
-        check(a4, 0, 'C', True)
+        check(a4, 0, 'C', True, True)
         a4.flags.writeable = False
-        check(a4, 0, 'C', False)
+        check(a4, 0, 'C', False, True)
 
     def test_structured_arrays(self):
-        def check(arr, dtype, ndim, layout):
+        def check(arr, dtype, ndim, layout, aligned):
             ty = typeof(arr)
             self.assertIsInstance(ty, types.Array)
             self.assertEqual(ty.dtype, dtype)
             self.assertEqual(ty.ndim, ndim)
             self.assertEqual(ty.layout, layout)
+            self.assertEqual(ty.aligned, aligned)
 
         dtype = np.dtype([('m', np.int32), ('n', 'S5')])
         rec_ty = numpy_support.from_struct_dtype(dtype)
 
         arr = np.empty(4, dtype=dtype)
-        check(arr, rec_ty, 1, "C")
+        check(arr, rec_ty, 1, "C", False)
         arr = np.recarray(4, dtype=dtype)
-        check(arr, rec_ty, 1, "C")
+        check(arr, rec_ty, 1, "C", False)
+
+        dtype = np.dtype([('m', np.int32), ('n', 'S5')], align=True)
+        rec_ty = numpy_support.from_struct_dtype(dtype)
+
+        # On Numpy 1.6, align=True doesn't align the itemsize
+        actual_aligned = numpy_support.version >= (1, 7)
+
+        arr = np.empty(4, dtype=dtype)
+        check(arr, rec_ty, 1, "C", actual_aligned)
+        arr = np.recarray(4, dtype=dtype)
+        check(arr, rec_ty, 1, "C", actual_aligned)
 
     @unittest.skipIf(sys.version_info < (2, 7),
                      "buffer protocol not supported on Python 2.6")
@@ -138,6 +156,10 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         ty = typeof(None)
         self.assertEqual(ty, types.none)
 
+    def test_ellipsis(self):
+        ty = typeof(Ellipsis)
+        self.assertEqual(ty, types.ellipsis)
+
     def test_str(self):
         ty = typeof("abc")
         self.assertEqual(ty, types.string)
@@ -151,6 +173,10 @@ class TestTypeof(ValueTypingTestBase, TestCase):
                                       types.Tuple((types.float64, types.intp))))
                          )
 
+    def test_lists(self):
+        v = [1.0] * 100
+        self.assertEqual(typeof(v), types.List(types.float64, reflected=True))
+
     def test_namedtuple(self):
         v = Point(1, 2)
         tp_point = typeof(v)
@@ -315,6 +341,8 @@ class TestFingerprint(TestCase):
         self.assertEqual(compute_fingerprint(arr[:1]), s)
         arr = np.empty(5, dtype=recordtype2)
         distinct.add(compute_fingerprint(arr))
+        arr = np.empty(5, dtype=recordtype3)
+        distinct.add(compute_fingerprint(arr))
 
         # np.recarray() is peculiar: it creates a new dtype instance in
         # its constructor; check that the fingerprint remains efficient
