@@ -31,6 +31,7 @@ no_pyobj_flags = Flags()
 enable_nrt_flags = Flags()
 enable_nrt_flags.set("nrt")
 
+
 def _unimplemented(func):
     """An 'expectedFailure' like decorator that only expects compilation errors
     caused by unimplemented functions that fail in no-python mode"""
@@ -86,16 +87,25 @@ def _make_binary_ufunc_op_usecase(ufunc_op):
     return fn
 
 
+def _make_inplace_ufunc_op_usecase(ufunc_op):
+    ldict = {}
+    exec("def fn(x,y):\n    x{0}y".format(ufunc_op), globals(), ldict)
+    fn = ldict["fn"]
+    fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
+    return fn
+
+
 def _as_dtype_value(tyargs, args):
     """Convert python values into numpy scalar objects.
     """
     return [np.dtype(str(ty)).type(val) for ty, val in zip(tyargs, args)]
 
 
-class TestUFuncs(MemoryLeakMixin, TestCase):
+
+class BaseUFuncTest(MemoryLeakMixin):
 
     def setUp(self):
-        super(TestUFuncs, self).setUp()
+        super(BaseUFuncTest, self).setUp()
         self.inputs = [
             (np.uint32(0), types.uint32),
             (np.uint32(1), types.uint32),
@@ -148,9 +158,13 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
                 output_type = types.Array(ty, 1, 'C')
         return output_type
 
+
+class TestUFuncs(BaseUFuncTest, TestCase):
+
     def unary_ufunc_test(self, ufunc, flags=enable_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
-                         int_output_type=None, float_output_type=None):
+                         int_output_type=None, float_output_type=None,
+                         kinds='ifc'):
         # Necessary to avoid some Numpy warnings being silenced, despite
         # the simplefilter() call below.
         self.reset_module_warnings(__name__)
@@ -168,6 +182,11 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
 
             if input_type in skip_inputs:
                 continue
+            # Some ufuncs don't allow all kinds of arguments, and implicit
+            # conversion has become stricter in 1.10.
+            if (numpy_support.strict_ufunc_typing and
+                input_operand.dtype.kind not in kinds):
+                continue
 
             output_type = self._determine_output_type(
                 input_type, int_output_type, float_output_type)
@@ -188,7 +207,6 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
             invalid_flag = False
             with warnings.catch_warnings(record=True) as warnlist:
                 warnings.simplefilter('always')
-
                 pyfunc(input_operand, expected)
 
                 warnmsg = "invalid value encountered"
@@ -225,30 +243,10 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
                                                expected.dtype, expected)
                         self.fail(msg)
 
-    def unary_op_test(self, operator, flags=enable_nrt_flags,
-                      skip_inputs=[], additional_inputs=[],
-                      int_output_type=None, float_output_type=None):
-        operator_func = _make_unary_ufunc_op_usecase(operator)
-        inputs = list(self.inputs)
-        inputs.extend(additional_inputs)
-        pyfunc = operator_func
-        for input_tuple in inputs:
-            input_operand, input_type = input_tuple
-
-            if ((input_type in skip_inputs) or
-                (not isinstance(input_type, types.Array))):
-                continue
-
-            cr = self.cache.compile(pyfunc, (input_type,),
-                                    flags=flags)
-            cfunc = cr.entry_point
-            expected = pyfunc(input_operand)
-            result = cfunc(input_operand)
-            np.testing.assert_array_almost_equal(expected, result)
-
     def binary_ufunc_test(self, ufunc, flags=enable_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
-                         int_output_type=None, float_output_type=None):
+                         int_output_type=None, float_output_type=None,
+                         kinds='ifc'):
 
         ufunc = _make_binary_ufunc_usecase(ufunc)
 
@@ -262,6 +260,12 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
             if input_type in skip_inputs:
                 continue
 
+            # Some ufuncs don't allow all kinds of arguments, and implicit
+            # conversion has become stricter in 1.10.
+            if (numpy_support.strict_ufunc_typing and
+                input_operand.dtype.kind not in kinds):
+                continue
+
             output_type = self._determine_output_type(
                 input_type, int_output_type, float_output_type)
 
@@ -281,55 +285,6 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
             pyfunc(input_operand, input_operand, expected)
             np.testing.assert_array_almost_equal(expected, result)
 
-    def binary_op_test(self, operator, flags=enable_nrt_flags,
-                       skip_inputs=[], additional_inputs=[],
-                       int_output_type=None, float_output_type=None,
-                       positive_rhs=False):
-        operator_func = _make_binary_ufunc_op_usecase(operator)
-        inputs = list(self.inputs)
-        inputs.extend(additional_inputs)
-        pyfunc = operator_func
-        for input_tuple in inputs:
-            input_operand1, input_type = input_tuple
-            input_dtype = numpy_support.as_dtype(
-                getattr(input_type, "dtype", input_type))
-            input_type1 = input_type
-
-            if input_type in skip_inputs:
-                continue
-
-            if positive_rhs:
-                zero = np.zeros(1, dtype=input_dtype)[0]
-            # If we only use two scalars, the code generator will not
-            # select the ufunctionalized operator, so we mix it up.
-            if isinstance(input_type, types.Array):
-                input_operand0 = input_operand1
-                input_type0 = input_type
-                if positive_rhs and np.any(input_operand1 < zero):
-                    continue
-            else:
-                input_operand0 = (np.random.random(10) * 100).astype(
-                    input_dtype)
-                input_type0 = typeof(input_operand0)
-                if positive_rhs and input_operand1 < zero:
-                    continue
-
-            cr = self.cache.compile(pyfunc, (input_type0, input_type1),
-                                    flags=flags)
-            cfunc = cr.entry_point
-            expected = pyfunc(input_operand0, input_operand1)
-            result = cfunc(input_operand0, input_operand1)
-            np.testing.assert_array_almost_equal(expected, result)
-
-    def binary_int_op_test(self, *args, **kws):
-        if 'skip_inputs' not in kws:
-            kws['skip_inputs'] = []
-        kws['skip_inputs'].extend([
-            types.float32, types.float64,
-            types.Array(types.float32, 1, 'C'),
-            types.Array(types.float64, 1, 'C')])
-        return self.binary_op_test(*args, **kws)
-
     def unary_int_ufunc_test(self, name=None, flags=enable_pyobj_flags):
         self.unary_ufunc_test(name, flags=flags,
             skip_inputs=[types.float32, types.float64,
@@ -376,16 +331,16 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
         self.test_divide_ufunc(flags=no_pyobj_flags)
 
     def test_logaddexp_ufunc(self):
-        self.binary_ufunc_test(np.logaddexp)
+        self.binary_ufunc_test(np.logaddexp, kinds='f')
 
     def test_logaddexp_ufunc_npm(self):
-        self.binary_ufunc_test(np.logaddexp, flags=no_pyobj_flags)
+        self.binary_ufunc_test(np.logaddexp, flags=no_pyobj_flags, kinds='f')
 
     def test_logaddexp2_ufunc(self):
-        self.binary_ufunc_test(np.logaddexp2)
+        self.binary_ufunc_test(np.logaddexp2, kinds='f')
 
     def test_logaddexp2_ufunc_npm(self):
-        self.binary_ufunc_test(np.logaddexp2, flags=no_pyobj_flags)
+        self.binary_ufunc_test(np.logaddexp2, flags=no_pyobj_flags, kinds='f')
 
     def test_true_divide_ufunc(self, flags=enable_pyobj_flags):
         self.binary_ufunc_test(np.true_divide, flags=flags, int_output_type=types.float64)
@@ -433,36 +388,32 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
     def test_fmod_ufunc_npm(self):
         self.test_fmod_ufunc(flags=no_pyobj_flags)
 
-    def test_abs_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.abs, flags=flags,
-            additional_inputs = [(np.iinfo(np.uint32).max, types.uint32),
-                                 (np.iinfo(np.uint64).max, types.uint64),
-                                 (np.finfo(np.float32).min, types.float32),
-                                 (np.finfo(np.float64).min, types.float64)
-                                 ])
+    def test_abs_ufunc(self, flags=enable_pyobj_flags, ufunc=np.abs):
+        self.unary_ufunc_test(ufunc, flags=flags,
+            additional_inputs = [
+                (np.uint32(np.iinfo(np.uint32).max), types.uint32),
+                (np.uint64(np.iinfo(np.uint64).max), types.uint64),
+                (np.float32(np.finfo(np.float32).min), types.float32),
+                (np.float64(np.finfo(np.float64).min), types.float64)
+                ])
 
     def test_abs_ufunc_npm(self):
         self.test_abs_ufunc(flags=no_pyobj_flags)
 
     def test_absolute_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.absolute, flags=flags,
-            additional_inputs = [(np.iinfo(np.uint32).max, types.uint32),
-                                 (np.iinfo(np.uint64).max, types.uint64),
-                                 (np.finfo(np.float32).min, types.float32),
-                                 (np.finfo(np.float64).min, types.float64)
-                                 ])
+        self.test_abs_ufunc(flags=flags, ufunc=np.absolute)
 
     def test_absolute_ufunc_npm(self):
         self.test_absolute_ufunc(flags=no_pyobj_flags)
 
     def test_fabs_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.fabs, flags=flags)
+        self.unary_ufunc_test(np.fabs, flags=flags, kinds='f')
 
     def test_fabs_ufunc_npm(self):
         self.test_fabs_ufunc(flags=no_pyobj_flags)
 
     def test_rint_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.rint, flags=flags)
+        self.unary_ufunc_test(np.rint, flags=flags, kinds='cf')
 
     def test_rint_ufunc_npm(self):
         self.test_rint_ufunc(flags=no_pyobj_flags)
@@ -480,49 +431,49 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
         self.test_conj_ufunc(flags=no_pyobj_flags)
 
     def test_exp_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.exp, flags=flags)
+        self.unary_ufunc_test(np.exp, flags=flags, kinds='cf')
 
     def test_exp_ufunc_npm(self):
         self.test_exp_ufunc(flags=no_pyobj_flags)
 
     def test_exp2_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.exp2, flags=flags)
+        self.unary_ufunc_test(np.exp2, flags=flags, kinds='cf')
 
     def test_exp2_ufunc_npm(self):
         self.test_exp2_ufunc(flags=no_pyobj_flags)
 
     def test_log_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.log, flags=flags)
+        self.unary_ufunc_test(np.log, flags=flags, kinds='cf')
 
     def test_log_ufunc_npm(self):
         self.test_log_ufunc(flags=no_pyobj_flags)
 
     def test_log2_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.log2, flags=flags)
+        self.unary_ufunc_test(np.log2, flags=flags, kinds='cf')
 
     def test_log2_ufunc_npm(self):
         self.test_log2_ufunc(flags=no_pyobj_flags)
 
     def test_log10_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.log10, flags=flags)
+        self.unary_ufunc_test(np.log10, flags=flags, kinds='cf')
 
     def test_log10_ufunc_npm(self):
         self.test_log10_ufunc(flags=no_pyobj_flags)
 
     def test_expm1_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.expm1, flags=flags)
+        self.unary_ufunc_test(np.expm1, flags=flags, kinds='cf')
 
     def test_expm1_ufunc_npm(self):
         self.test_expm1_ufunc(flags=no_pyobj_flags)
 
     def test_log1p_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.log1p, flags=flags)
+        self.unary_ufunc_test(np.log1p, flags=flags, kinds='cf')
 
     def test_log1p_ufunc_npm(self):
         self.test_log1p_ufunc(flags=no_pyobj_flags)
 
     def test_sqrt_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.sqrt, flags=flags)
+        self.unary_ufunc_test(np.sqrt, flags=flags, kinds='cf')
 
     def test_sqrt_ufunc_npm(self):
         self.test_sqrt_ufunc(flags=no_pyobj_flags)
@@ -557,79 +508,79 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
     # Trigonometric Functions
 
     def test_sin_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.sin, flags=flags)
+        self.unary_ufunc_test(np.sin, flags=flags, kinds='cf')
 
     def test_sin_ufunc_npm(self):
         self.test_sin_ufunc(flags=no_pyobj_flags)
 
     def test_cos_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.cos, flags=flags)
+        self.unary_ufunc_test(np.cos, flags=flags, kinds='cf')
 
     def test_cos_ufunc_npm(self):
         self.test_cos_ufunc(flags=no_pyobj_flags)
 
     def test_tan_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.tan, flags=flags)
+        self.unary_ufunc_test(np.tan, flags=flags, kinds='cf')
 
     def test_tan_ufunc_npm(self):
         self.test_tan_ufunc(flags=no_pyobj_flags)
 
     def test_arcsin_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.arcsin, flags=flags)
+        self.unary_ufunc_test(np.arcsin, flags=flags, kinds='cf')
 
     def test_arcsin_ufunc_npm(self):
         self.test_arcsin_ufunc(flags=no_pyobj_flags)
 
     def test_arccos_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.arccos, flags=flags)
+        self.unary_ufunc_test(np.arccos, flags=flags, kinds='cf')
 
     def test_arccos_ufunc_npm(self):
         self.test_arccos_ufunc(flags=no_pyobj_flags)
 
     def test_arctan_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.arctan, flags=flags)
+        self.unary_ufunc_test(np.arctan, flags=flags, kinds='cf')
 
     def test_arctan_ufunc_npm(self):
         self.test_arctan_ufunc(flags=no_pyobj_flags)
 
     def test_arctan2_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test(np.arctan2, flags=flags)
+        self.binary_ufunc_test(np.arctan2, flags=flags, kinds='cf')
 
     def test_arctan2_ufunc_npm(self):
         self.test_arctan2_ufunc(flags=no_pyobj_flags)
 
     def test_hypot_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test(np.hypot)
+        self.binary_ufunc_test(np.hypot, kinds='f')
 
     def test_hypot_ufunc_npm(self):
         self.test_hypot_ufunc(flags=no_pyobj_flags)
 
     def test_sinh_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.sinh, flags=flags)
+        self.unary_ufunc_test(np.sinh, flags=flags, kinds='cf')
 
     def test_sinh_ufunc_npm(self):
         self.test_sinh_ufunc(flags=no_pyobj_flags)
 
     def test_cosh_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.cosh, flags=flags)
+        self.unary_ufunc_test(np.cosh, flags=flags, kinds='cf')
 
     def test_cosh_ufunc_npm(self):
         self.test_cosh_ufunc(flags=no_pyobj_flags)
 
     def test_tanh_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.tanh, flags=flags)
+        self.unary_ufunc_test(np.tanh, flags=flags, kinds='cf')
 
     def test_tanh_ufunc_npm(self):
         self.test_tanh_ufunc(flags=no_pyobj_flags)
 
     def test_arcsinh_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.arcsinh, flags=flags)
+        self.unary_ufunc_test(np.arcsinh, flags=flags, kinds='cf')
 
     def test_arcsinh_ufunc_npm(self):
         self.test_arcsinh_ufunc(flags=no_pyobj_flags)
 
     def test_arccosh_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.arccosh, flags=flags)
+        self.unary_ufunc_test(np.arccosh, flags=flags, kinds='cf')
 
     def test_arccosh_ufunc_npm(self):
         self.test_arccosh_ufunc(flags=no_pyobj_flags)
@@ -648,31 +599,32 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
                    types.Array(types.uint64, 1, 'C'), types.uint64,
                    types.Array(types.int64, 1, 'C'), types.int64]
 
-        self.unary_ufunc_test(np.arctanh, skip_inputs=to_skip, flags=flags)
+        self.unary_ufunc_test(np.arctanh, skip_inputs=to_skip, flags=flags,
+                              kinds='cf')
 
     def test_arctanh_ufunc_npm(self):
         self.test_arctanh_ufunc(flags=no_pyobj_flags)
 
     def test_deg2rad_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.deg2rad, flags=flags)
+        self.unary_ufunc_test(np.deg2rad, flags=flags, kinds='f')
 
     def test_deg2rad_ufunc_npm(self):
         self.test_deg2rad_ufunc(flags=no_pyobj_flags)
 
     def test_rad2deg_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.rad2deg, flags=flags)
+        self.unary_ufunc_test(np.rad2deg, flags=flags, kinds='f')
 
     def test_rad2deg_ufunc_npm(self):
         self.test_rad2deg_ufunc(flags=no_pyobj_flags)
 
     def test_degrees_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.degrees, flags=flags)
+        self.unary_ufunc_test(np.degrees, flags=flags, kinds='f')
 
     def test_degrees_ufunc_npm(self):
         self.test_degrees_ufunc(flags=no_pyobj_flags)
 
     def test_radians_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.radians, flags=flags)
+        self.unary_ufunc_test(np.radians, flags=flags, kinds='f')
 
     def test_radians_ufunc_npm(self):
         self.test_radians_ufunc(flags=no_pyobj_flags)
@@ -836,19 +788,19 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
         self.test_signbit_ufunc(flags=no_pyobj_flags)
 
     def test_copysign_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test(np.copysign, flags=flags)
+        self.binary_ufunc_test(np.copysign, flags=flags, kinds='f')
 
     def test_copysign_ufunc_npm(self):
         self.test_copysign_ufunc(flags=no_pyobj_flags)
 
     def test_nextafter_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test(np.nextafter, flags=flags)
+        self.binary_ufunc_test(np.nextafter, flags=flags, kinds='f')
 
     def test_nextafter_ufunc_npm(self):
         self.test_nextafter_ufunc(flags=no_pyobj_flags)
 
     def test_modf_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.modf, flags=flags)
+        self.unary_ufunc_test(np.modf, flags=flags, kinds='f')
 
     @_unimplemented
     def test_modf_ufunc_npm(self):
@@ -859,32 +811,32 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
     #       However, this ufunc is tested by the TestLoopTypes test classes.
 
     def test_frexp_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.frexp, flags=flags)
+        self.unary_ufunc_test(np.frexp, flags=flags, kinds='f')
 
     @_unimplemented
     def test_frexp_ufunc_npm(self):
         self.test_frexp_ufunc(flags=no_pyobj_flags)
 
     def test_floor_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.floor, flags=flags)
+        self.unary_ufunc_test(np.floor, flags=flags, kinds='f')
 
     def test_floor_ufunc_npm(self):
         self.test_floor_ufunc(flags=no_pyobj_flags)
 
     def test_ceil_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.ceil, flags=flags)
+        self.unary_ufunc_test(np.ceil, flags=flags, kinds='f')
 
     def test_ceil_ufunc_npm(self):
         self.test_ceil_ufunc(flags=no_pyobj_flags)
 
     def test_trunc_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.trunc, flags=flags)
+        self.unary_ufunc_test(np.trunc, flags=flags, kinds='f')
 
     def test_trunc_ufunc_npm(self):
         self.test_trunc_ufunc(flags=no_pyobj_flags)
 
     def test_spacing_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.spacing, flags=flags)
+        self.unary_ufunc_test(np.spacing, flags=flags, kinds='f')
 
     def test_spacing_ufunc_npm(self):
         self.test_spacing_ufunc(flags=no_pyobj_flags)
@@ -992,8 +944,8 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
                                 np.allclose(result, expected))
 
     def test_mixed_types(self):
-        self.binary_ufunc_mixed_types_test(np.divide, flags=no_pyobj_flags)
-
+        if not numpy_support.strict_ufunc_typing:
+            self.binary_ufunc_mixed_types_test(np.divide, flags=no_pyobj_flags)
 
     def test_broadcasting(self):
 
@@ -1101,36 +1053,195 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
             result = cfunc(x, y)
             np.testing.assert_array_equal(expected, result)
 
-    def test_implicit_output_layout(self):
+    def test_implicit_output_layout_binary(self):
         def pyfunc(a0, a1):
             return np.add(a0, a1)
+
+        # C layout
         X = np.linspace(0, 1, 20).reshape(4, 5)
+        # F layout
         Y = np.array(X, order='F')
+        # A layout
+        Z = X.reshape(5, 4).T[0]
+
         Xty = typeof(X)
         assert X.flags.c_contiguous and Xty.layout == 'C'
         Yty = typeof(Y)
         assert Y.flags.f_contiguous and Yty.layout == 'F'
+        Zty = typeof(Z)
+        assert Zty.layout == 'A'
+        assert not Z.flags.c_contiguous
+        assert not Z.flags.f_contiguous
 
-        cr0 = self.cache.compile(pyfunc, (Xty, Yty), flags=enable_nrt_flags)
-        expected0 = np.add(X, Y)
-        result0 = cr0.entry_point(X, Y)
-        self.assertEqual(expected0.flags.c_contiguous,
-                         result0.flags.c_contiguous)
-        self.assertEqual(expected0.flags.f_contiguous,
-                         result0.flags.f_contiguous)
-        np.testing.assert_array_equal(expected0, result0)
+        testcases = list(itertools.permutations([X, Y, Z], 2))
+        testcases += [(X, X)]
+        testcases += [(Y, Y)]
+        testcases += [(Z, Z)]
 
-        cr1 = self.cache.compile(pyfunc, (Yty, Yty), flags=enable_nrt_flags)
-        expected1 = np.add(Y, Y)
-        result1 = cr1.entry_point(Y, Y)
-        self.assertEqual(expected1.flags.c_contiguous,
-                         result1.flags.c_contiguous)
-        self.assertEqual(expected1.flags.f_contiguous,
-                         result1.flags.f_contiguous)
-        np.testing.assert_array_equal(expected1, result1)
+        for arg0, arg1 in testcases:
+            cr = self.cache.compile(pyfunc, (typeof(arg0), typeof(arg1)),
+                                    flags=enable_nrt_flags)
+            expected = pyfunc(arg0, arg1)
+            result = cr.entry_point(arg0, arg1)
+
+            self.assertEqual(expected.flags.c_contiguous,
+                             result.flags.c_contiguous)
+            self.assertEqual(expected.flags.f_contiguous,
+                             result.flags.f_contiguous)
+            np.testing.assert_array_equal(expected, result)
+
+    def test_implicit_output_layout_unary(self):
+        def pyfunc(a0):
+            return np.sqrt(a0)
+
+        # C layout
+        X = np.linspace(0, 1, 20).reshape(4, 5)
+        # F layout
+        Y = np.array(X, order='F')
+        # A layout
+        Z = X.reshape(5, 4).T[0]
+
+        Xty = typeof(X)
+        assert X.flags.c_contiguous and Xty.layout == 'C'
+        Yty = typeof(Y)
+        assert Y.flags.f_contiguous and Yty.layout == 'F'
+        Zty = typeof(Z)
+        assert Zty.layout == 'A'
+        assert not Z.flags.c_contiguous
+        assert not Z.flags.f_contiguous
+
+        for arg0 in [X, Y, Z]:
+            cr = self.cache.compile(pyfunc, (typeof(arg0),),
+                                    flags=enable_nrt_flags)
+            expected = pyfunc(arg0)
+            result = cr.entry_point(arg0)
+
+            self.assertEqual(expected.flags.c_contiguous,
+                             result.flags.c_contiguous)
+            self.assertEqual(expected.flags.f_contiguous,
+                             result.flags.f_contiguous)
+            np.testing.assert_array_equal(expected, result)
+
+
+
+class TestArrayOperators(BaseUFuncTest, TestCase):
+
+    def unary_op_test(self, operator, flags=enable_nrt_flags,
+                      skip_inputs=[], additional_inputs=[],
+                      int_output_type=None, float_output_type=None):
+        operator_func = _make_unary_ufunc_op_usecase(operator)
+        inputs = list(self.inputs)
+        inputs.extend(additional_inputs)
+        pyfunc = operator_func
+        for input_tuple in inputs:
+            input_operand, input_type = input_tuple
+
+            if ((input_type in skip_inputs) or
+                (not isinstance(input_type, types.Array))):
+                continue
+
+            cr = self.cache.compile(pyfunc, (input_type,),
+                                    flags=flags)
+            cfunc = cr.entry_point
+            expected = pyfunc(input_operand)
+            result = cfunc(input_operand)
+            np.testing.assert_array_almost_equal(expected, result)
+
+    def binary_op_test(self, operator, flags=enable_nrt_flags,
+                       skip_inputs=[], additional_inputs=[],
+                       int_output_type=None, float_output_type=None,
+                       positive_rhs=False):
+        operator_func = _make_binary_ufunc_op_usecase(operator)
+        inputs = list(self.inputs)
+        inputs.extend(additional_inputs)
+        pyfunc = operator_func
+        for input_tuple in inputs:
+            input_operand1, input_type = input_tuple
+            input_dtype = numpy_support.as_dtype(
+                getattr(input_type, "dtype", input_type))
+            input_type1 = input_type
+
+            if input_type in skip_inputs:
+                continue
+
+            if positive_rhs:
+                zero = np.zeros(1, dtype=input_dtype)[0]
+            # If we only use two scalars, the code generator will not
+            # select the ufunctionalized operator, so we mix it up.
+            if isinstance(input_type, types.Array):
+                input_operand0 = input_operand1
+                input_type0 = input_type
+                if positive_rhs and np.any(input_operand1 < zero):
+                    continue
+            else:
+                input_operand0 = (np.random.random(10) * 100).astype(
+                    input_dtype)
+                input_type0 = typeof(input_operand0)
+                if positive_rhs and input_operand1 < zero:
+                    continue
+
+            cr = self.cache.compile(pyfunc, (input_type0, input_type1),
+                                    flags=flags)
+            cfunc = cr.entry_point
+            expected = pyfunc(input_operand0, input_operand1)
+            result = cfunc(input_operand0, input_operand1)
+            np.testing.assert_array_almost_equal(expected, result)
+
+    def binary_int_op_test(self, *args, **kws):
+        if 'skip_inputs' not in kws:
+            kws['skip_inputs'] = []
+        kws['skip_inputs'].extend([
+            types.float32, types.float64,
+            types.Array(types.float32, 1, 'C'),
+            types.Array(types.float64, 1, 'C')])
+        return self.binary_op_test(*args, **kws)
+
+    def _make_arrays(self, dtypes):
+        for dtype in dtypes:
+            yield np.linspace(0, 5, 3).astype(dtype)
+            yield np.linspace(1, 6, 3).astype(dtype)
+
+    def inplace_op_test(self, operator, lhs_values, rhs_values,
+                        lhs_dtypes, rhs_dtypes):
+        operator_func = _make_inplace_ufunc_op_usecase(operator)
+        pyfunc = operator_func
+
+        # The left operand can only be an array, while the right operand
+        # can be either an array or a scalar
+        lhs_inputs = [np.array(lhs_values, dtype=dtype)
+                      for dtype in lhs_dtypes]
+
+        rhs_arrays = [np.array(rhs_values, dtype=dtype)
+                      for dtype in rhs_dtypes]
+        rhs_scalars = [dtype(v) for v in rhs_values for dtype in rhs_dtypes]
+        rhs_inputs = rhs_arrays + rhs_scalars
+
+        for lhs, rhs in itertools.product(lhs_inputs, rhs_inputs):
+            lhs_type = typeof(lhs)
+            rhs_type = typeof(rhs)
+            cr = self.cache.compile(pyfunc, (lhs_type, rhs_type),
+                                    flags=no_pyobj_flags)
+            cfunc = cr.entry_point
+            expected = lhs.copy()
+            pyfunc(expected, rhs)
+            got = lhs.copy()
+            cfunc(got, rhs)
+            self.assertPreciseEqual(got, expected)
+
+    def inplace_float_op_test(self, operator, lhs_values, rhs_values):
+        # Also accept integer inputs for the right operand (they should
+        # be converted to float).
+        return self.inplace_op_test(operator, lhs_values, rhs_values,
+                                    (np.float32, np.float64),
+                                    (np.float32, np.float64, np.int64))
+
+    def inplace_int_op_test(self, operator, lhs_values, rhs_values):
+        return self.inplace_op_test(operator, lhs_values, rhs_values,
+                                    (np.int16, np.int32, np.int64),
+                                    (np.int16, np.uint32))
 
     # ____________________________________________________________
-    # Array operators
+    # Unary operators
 
     def test_unary_positive_array_op(self):
         self.unary_op_test('+')
@@ -1144,6 +1255,70 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
             types.Array(types.float32, 1, 'C'),
             types.Array(types.float64, 1, 'C')])
 
+    # ____________________________________________________________
+    # Inplace operators
+
+    def test_inplace_add(self):
+        self.inplace_float_op_test('+=', [-1, 1.5, 3], [-5, 0, 2.5])
+
+    def test_inplace_sub(self):
+        self.inplace_float_op_test('-=', [-1, 1.5, 3], [-5, 0, 2.5])
+
+    def test_inplace_mul(self):
+        self.inplace_float_op_test('*=', [-1, 1.5, 3], [-5, 0, 2.5])
+
+    def test_inplace_floordiv(self):
+        self.inplace_float_op_test('//=', [-1, 1.5, 3], [-5, 0, 2.5])
+
+    def test_inplace_div(self):
+        self.inplace_float_op_test('/=', [-1, 1.5, 3], [-5, 0, 2.5])
+
+    def test_inplace_remainder(self):
+        self.inplace_float_op_test('%=', [-1, 1.5, 3], [-5, 2, 2.5])
+
+    def test_inplace_pow(self):
+        self.inplace_float_op_test('**=', [-1, 1.5, 3], [-5, 2, 2.5])
+
+    def test_inplace_and(self):
+        self.inplace_int_op_test('&=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+
+    def test_inplace_or(self):
+        self.inplace_int_op_test('|=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+
+    def test_inplace_xor(self):
+        self.inplace_int_op_test('^=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+
+    def test_inplace_lshift(self):
+        self.inplace_int_op_test('<<=', [0, 5, -10, -51], [0, 1, 4, 14])
+
+    def test_inplace_rshift(self):
+        self.inplace_int_op_test('>>=', [0, 5, -10, -51], [0, 1, 4, 14])
+
+    def test_unary_positive_array_op(self):
+        '''
+        Verify that the unary positive operator copies values, and doesn't
+        just alias to the input array (mirrors normal Numpy/Python
+        interaction behavior).
+        '''
+        # Test originally from @gmarkall
+        def f(a1):
+            a2 = +a1
+            a1[0] = 3
+            a2[1] = 4
+            return a2
+
+        a1 = np.zeros(10)
+        a2 = f(a1)
+        self.assertTrue(a1[0] != a2[0] and a1[1] != a2[1])
+        a3 = np.zeros(10)
+        a4 = njit(f)(a3)
+        self.assertTrue(a3[0] != a4[0] and a3[1] != a4[1])
+        np.testing.assert_array_equal(a1, a3)
+        np.testing.assert_array_equal(a2, a4)
+
+    # ____________________________________________________________
+    # Binary operators
+
     def test_add_array_op(self):
         self.binary_op_test('+')
 
@@ -1201,28 +1376,6 @@ class TestUFuncs(MemoryLeakMixin, TestCase):
     def test_not_equal_array_op(self):
         self.binary_op_test('!=')
 
-    def test_unary_positive_array_op(self):
-        '''
-        Verify that the unary positive operator copies values, and doesn't
-        just alias to the input array (mirrors normal Numpy/Python
-        interaction behavior).
-        '''
-        # Test originally from @gmarkall
-        def f(a1):
-            a2 = +a1
-            a1[0] = 3
-            a2[1] = 4
-            return a2
-
-        a1 = np.zeros(10)
-        a2 = f(a1)
-        self.assertTrue(a1[0] != a2[0] and a1[1] != a2[1])
-        a3 = np.zeros(10)
-        a4 = njit(f)(a3)
-        self.assertTrue(a3[0] != a4[0] and a3[1] != a4[1])
-        np.testing.assert_array_equal(a1, a3)
-        np.testing.assert_array_equal(a2, a4)
-
 
 class TestScalarUFuncs(TestCase):
     """check the machinery of ufuncs works when the result is an scalar.
@@ -1371,6 +1524,7 @@ class _TestLoopTypes(TestCase):
     _ufuncs = all_ufuncs[:]
     # Have their own test classes
     _ufuncs.remove(np.left_shift)
+    _ufuncs.remove(np.right_shift)
     _ufuncs.remove(np.reciprocal)
     _ufuncs.remove(np.power)
     _compile_flags = enable_pyobj_flags
@@ -1380,6 +1534,10 @@ class _TestLoopTypes(TestCase):
     if is_on_numpy_16:
         _skip_types += 'Mm'
 
+    _ulps = {('arccos', 'F'): 2,
+             ('tanh', 'F'): 2,
+             }
+
     def _arg_for_type(self, a_letter_type, index=0):
         """return a suitable array argument for testing the letter type"""
         if a_letter_type in 'bhilq':
@@ -1451,12 +1609,15 @@ class _TestLoopTypes(TestCase):
         # Check each array (including inputs, to ensure they weren't
         # mutated).
         for c_arg, py_arg in zip(c_args, py_args):
-            prec = 'single' if c_arg.dtype.char in 'fF' else 'exact'
-            prec = 'double' if c_arg.dtype.char in 'dD' else prec
+            typechar = c_arg.dtype.char
+            ulps = self._ulps.get((ufunc.__name__, typechar), 1)
+            prec = 'single' if typechar in 'fF' else 'exact'
+            prec = 'double' if typechar in 'dD' else prec
             msg = '\n'.join(["ufunc '{0}' arrays differ ({1}):",
                              "args: {2}", "expected {3}", "got {4}"])
             msg = msg.format(ufunc.__name__, c_args, prec, py_arg, c_arg)
-            self.assertPreciseEqual(py_arg, c_arg, prec=prec, msg=msg)
+            self.assertPreciseEqual(py_arg, c_arg, prec=prec, msg=msg,
+                                    ulps=ulps)
 
     @classmethod
     def _check_ufunc_loops(cls, ufunc):
@@ -1633,8 +1794,9 @@ class TestLoopTypesDatetimeNoPython(_TestLoopTypes):
         # heterogenous inputs
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[s]', 'm8[m]', 'm8[s]'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8[s]', 'm8[s]'])
-        self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8', 'm8[m]'])
-        self._check_ufunc_with_dtypes(fn, ufunc, ['m8', 'm8[m]', 'm8[m]'])
+        if not numpy_support.strict_ufunc_typing:
+            self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8', 'm8[m]'])
+            self._check_ufunc_with_dtypes(fn, ufunc, ['m8', 'm8[m]', 'm8[m]'])
         # heterogenous inputs, scaled output
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[s]', 'm8[m]', 'm8[ms]'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8[s]', 'm8[ms]'])
@@ -1693,8 +1855,9 @@ class TestLoopTypesDatetimeNoPython(_TestLoopTypes):
         # timedelta
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8[s]', '?'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[s]', 'm8[m]', '?'])
-        self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8', '?'])
-        self._check_ufunc_with_dtypes(fn, ufunc, ['m8', 'm8[m]', '?'])
+        if not numpy_support.strict_ufunc_typing:
+            self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8', '?'])
+            self._check_ufunc_with_dtypes(fn, ufunc, ['m8', 'm8[m]', '?'])
         # datetime
         self._check_ufunc_with_dtypes(fn, ufunc, ['M8[m]', 'M8[s]', '?'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['M8[s]', 'M8[m]', '?'])
