@@ -6,5 +6,5 @@ import sys
 import numba.testing as testing
 
 if __name__ == "__main__":
-    result = testing.test()
+    result = testing.run_tests(sys.argv)
     sys.exit(0 if result else 1)
